@@ -303,6 +303,8 @@ proc redraw_new_style {} {
     set recipoktime_total 0
     set busyslaves_total 0
     set totalslaves_total 0
+    set busyslaves_active 0
+    set totalslaves_active 0
     set num_machines 0
 
     set num_graphs 1
@@ -319,6 +321,36 @@ proc redraw_new_style {} {
 	incr num_graphs 2
     }
 
+    set wid [winfo width .top.clabels]
+    .top.clabels delete all
+    set spacing [expr $wid / (1.0 * $num_graphs)]
+    set x [expr $spacing / 2.0]
+    .top.clabels create text $x 2 -anchor n -fill "#A00000" -text "Busy Slaves"
+    if {$NewStyleShowScans} {
+	set x [expr $x + $spacing]
+	.top.clabels create text $x 2 -anchor n -fill "#00A000" -text "Scans/d"
+	set x [expr $x + $spacing]
+	.top.clabels create text $x 2 -anchor n -fill "#0000A0" -text "ms/scan"
+    }
+    if {$NewStyleShowRelayoks} {
+	set x [expr $x + $spacing]
+	.top.clabels create text $x 2 -anchor n -fill "#808000" -text "Relays/d"
+	set x [expr $x + $spacing]
+	.top.clabels create text $x 2 -anchor n -fill "#008080" -text "ms/relay"
+    }
+    if {$NewStyleShowSenderoks} {
+	set x [expr $x + $spacing]
+	.top.clabels create text $x 2 -anchor n -fill "#808080" -text "Senders/d"
+	set x [expr $x + $spacing]
+	.top.clabels create text $x 2 -anchor n -fill "#800080" -text "ms/sender"
+    }
+    if {$NewStyleShowRecipoks} {
+	set x [expr $x + $spacing]
+	.top.clabels create text $x 2 -anchor n -fill "#008000" -text "Recips/d"
+	set x [expr $x + $spacing]
+	.top.clabels create text $x 2 -anchor n -fill "#000000" -text "ms/recip"
+    }
+
     foreach m $Machines {
 	set mach [lindex $m 0]
 	if {![info exists Data($mach,busy)]} {
@@ -334,16 +366,23 @@ proc redraw_new_style {} {
 	set busy $Data($mach,busy_snap)
 	set totalslaves $Data($mach,total_slaves)
 
+	# Only update slave counts for machines that are actually doing something
+	if {$Data($mach,scanspersec_snap) > 0 || $Data($mach,relayokspersec_snap) > 0 || $Data($mach,senderokspersec_snap) > 0 || $Data($mach,recipokspersec_snap) > 0} {
+	    set totalslaves_active [expr $totalslaves_active + $totalslaves]
+	    set busyslaves_active [expr $busyslaves_active + 1.0*$busy]
+	}
+
 	set totalslaves_total [expr $totalslaves_total + $totalslaves]
-	set busyslaves_total [expr $busyslaves_total + $busy]
-	set scanspersec_total [expr $scanspersec_total + $Data($mach,scanspersec_snap)]
-	set scantime_total [expr $scantime_total + $Data($mach,scantime_snap)]
-	set relayokspersec_total [expr $relayokspersec_total + $Data($mach,relayokspersec_snap)]
-	set relayoktime_total [expr $relayoktime_total + $Data($mach,relayoktime_snap)]
-	set senderokspersec_total [expr $senderokspersec_total + $Data($mach,senderokspersec_snap)]
-	set senderoktime_total [expr $senderoktime_total + $Data($mach,senderoktime_snap)]
-	set recipokspersec_total [expr $recipokspersec_total + $Data($mach,recipokspersec_snap)]
-	set recipoktime_total [expr $recipoktime_total + $Data($mach,recipoktime_snap)]
+	set busyslaves_total [expr $busyslaves_total + 1.0*$busy]
+
+	set scanspersec_total [expr $scanspersec_total + 1.0*$Data($mach,scanspersec_snap)]
+	set scantime_total [expr $scantime_total + (1.0*$Data($mach,scanspersec_snap) * $Data($mach,scantime_snap))]
+	set relayokspersec_total [expr $relayokspersec_total + 1.0*$Data($mach,relayokspersec_snap)]
+	set relayoktime_total [expr $relayoktime_total + (1.0*$Data($mach,relayokspersec_snap) * $Data($mach,relayoktime_snap))]
+	set senderokspersec_total [expr $senderokspersec_total + 1.0*$Data($mach,senderokspersec_snap)]
+	set senderoktime_total [expr $senderoktime_total + (1.0*$Data($mach,senderokspersec_snap) * $Data($mach,senderoktime_snap))]
+	set recipokspersec_total [expr $recipokspersec_total + 1.0*$Data($mach,recipokspersec_snap)]
+	set recipoktime_total [expr $recipoktime_total + (1.0*$Data($mach,recipokspersec_snap) * $Data($mach,recipoktime_snap))]
 
 	set graph 0
 	set Data($mach,busy) [graph [expr $graph / (1.0 * $num_graphs)] [expr ($graph + 1.0) / (1.0 * $num_graphs)] 0 $Data($mach,total_slaves) $Data($mach,busy) $index "Busy" $graph "#A00000" 1]
@@ -352,7 +391,16 @@ proc redraw_new_style {} {
 	} else {
 	    set pctbusy 100
 	}
+	if {$pctbusy < 80} {
+	    .top.busy$index configure -background #D9D9D9 -foreground "#A00000"
+	} elseif {$pctbusy < 90} {
+	    .top.busy$index configure -background #C0C000 -foreground "#A00000"
+	} else {
+	    .top.busy$index configure -background #C00000 -foreground "#000000"
+	}
+
 	.top.busy$index configure -text "$busy/$totalslaves\n$pctbusy%"
+
 	incr graph
 
 	if {$NewStyleShowScans} {
@@ -363,8 +411,13 @@ proc redraw_new_style {} {
 	    set s $Data($mach,scanspersec_snap)
 	    set h [human_number [expr $s * 3600]]
 	    set d [human_number [expr $s * 86400]]
-	    .top.scanspersec$index configure -text [format "%.2f\n%s/h\n%s/d" $s $h $d]
-	    .top.scantime$index configure -text $Data($mach,scantime_snap)
+	    if {$s == 0} {
+		.top.scanspersec$index configure -text "-"
+		.top.scantime$index configure -text "-"
+	    } else {
+		.top.scanspersec$index configure -text [format "%.2f\n%s/h\n%s/d" $s $h $d]
+		.top.scantime$index configure -text $Data($mach,scantime_snap)
+	    }
 	} else {
 	    set Data($mach,scanspersec) {}
 	    set Data($mach,scantime) {}
@@ -378,8 +431,13 @@ proc redraw_new_style {} {
 	    set s $Data($mach,relayokspersec_snap)
 	    set h [human_number [expr $s * 3600]]
 	    set d [human_number [expr $s * 86400]]
-	    .top.relayspersec$index configure -text [format "%.2f\n%s/h\n%s/d" $s $h $d]
-	    .top.relaytime$index configure -text $Data($mach,relayoktime_snap)
+	    if {$s == 0} {
+		.top.relayspersec$index configure -text "-"
+		.top.relaytime$index configure -text "-"
+	    } else {
+		.top.relayspersec$index configure -text [format "%.2f\n%s/h\n%s/d" $s $h $d]
+		.top.relaytime$index configure -text $Data($mach,relayoktime_snap)
+	    }
 	} else {
 	    set Data($mach,relayokspersec) {}
 	    set Data($mach,relayoktime) {}
@@ -392,8 +450,13 @@ proc redraw_new_style {} {
 	    set s $Data($mach,senderokspersec_snap)
 	    set h [human_number [expr $s * 3600]]
 	    set d [human_number [expr $s * 86400]]
-	    .top.senderspersec$index configure -text [format "%.2f\n%s/h\n%s/d" $s $h $d]
-	    .top.sendertime$index configure -text $Data($mach,senderoktime_snap)
+	    if {$s == 0} {
+		.top.senderspersec$index configure -text "-"
+		.top.sendertime$index configure -text "-"
+	    } else {
+		.top.senderspersec$index configure -text [format "%.2f\n%s/h\n%s/d" $s $h $d]
+		.top.sendertime$index configure -text $Data($mach,senderoktime_snap)
+	    }
 	} else {
 	    set Data($mach,senderokspersec) {}
 	    set Data($mach,senderoktime) {}
@@ -406,8 +469,13 @@ proc redraw_new_style {} {
 	    set s $Data($mach,recipokspersec_snap)
 	    set h [human_number [expr $s * 3600]]
 	    set d [human_number [expr $s * 86400]]
-	    .top.recipspersec$index configure -text [format "%.2f\n%s/h\n%s/d" $s $h $d]
-	    .top.reciptime$index configure -text $Data($mach,recipoktime_snap)
+	    if {$s == 0} {
+		.top.recipspersec$index configure -text "-"
+		.top.reciptime$index configure -text "-"
+	    } else {
+		.top.recipspersec$index configure -text [format "%.2f\n%s/h\n%s/d" $s $h $d]
+		.top.reciptime$index configure -text $Data($mach,recipoktime_snap)
+	    }
 	} else {
 	    set Data($mach,recipokspersec) {}
 	    set Data($mach,recipoktime) {}
@@ -417,23 +485,34 @@ proc redraw_new_style {} {
     lappend TotalData(busy) $busyslaves_total
     lappend TotalData(total) $totalslaves_total
     lappend TotalData(scanspersec) $scanspersec_total
-    lappend TotalData(scantime) [expr $scantime_total / (1.0 * $index)]
+    set scantime_avg [expr $scantime_total / ($scanspersec_total + 0.000000001)]
+    set relayoktime_avg [expr $relayoktime_total / ($relayokspersec_total + 0.000000001)]
+    set senderoktime_avg [expr $senderoktime_total / ($senderokspersec_total + 0.000000001)]
+    set recipoktime_avg [expr $recipoktime_total / ($recipokspersec_total + 0.000000001)]
+    lappend TotalData(scantime) $scantime_avg
     lappend TotalData(relayokspersec) $relayokspersec_total
-    lappend TotalData(relayoktime) [expr $relayoktime_total / (1.0 * $index)]
+    lappend TotalData(relayoktime) $relayoktime_avg
     lappend TotalData(senderokspersec) $senderokspersec_total
-    lappend TotalData(senderoktime) [expr $senderoktime_total / (1.0 * $index)]
+    lappend TotalData(senderoktime) $senderoktime_avg
     lappend TotalData(recipokspersec) $recipokspersec_total
-    lappend TotalData(recipoktime) [expr $recipoktime_total / (1.0 * $index)]
+    lappend TotalData(recipoktime) $recipoktime_avg
 
     set graph 0
     set TotalData(busy) [graph [expr $graph / (1.0 * $num_graphs)] [expr ($graph + 1.0) / (1.0 * $num_graphs)] 0 $totalslaves_total $TotalData(busy) $index "Busy" $graph "#A00000" 1]
     if {$totalslaves_total > 0} {
+	set busyslaves_total [expr int($busyslaves_total)]
 	set pctbusy [expr int((1.0 * $busyslaves_total) / (1.0 * $totalslaves_total) * 100)]
     } else {
 	set pctbusy 100
     }
 
-    .top.busytotal configure -text "$busyslaves_total/$totalslaves_total\n$pctbusy%"
+    if {$totalslaves_active > 0} {
+	set busyslaves_active [expr int($busyslaves_active)]
+	set apct [expr int((1.0 * $busyslaves_active) / (1.0 * $totalslaves_active) * 100)]
+    } else {
+	set apct 100
+    }
+    .top.busytotal configure -text "$busyslaves_total/$totalslaves_total\n$pctbusy%\n$busyslaves_active/$totalslaves_active\n$apct%"
     incr graph
 
     if {$NewStyleShowScans} {
@@ -444,7 +523,7 @@ proc redraw_new_style {} {
 	set h [human_number [expr $scanspersec_total * 3600]]
 	set d [human_number [expr $scanspersec_total * 86400]]
 	.top.scanspersectotal configure -text [format "%.2f\n%s/h\n%s/d" $scanspersec_total $h $d]
-	.top.avgscantime configure -text [format "%.0f" [expr $scantime_total / (1.0 * $index)]]
+	.top.avgscantime configure -text [format "%.0f" $scantime_avg]
     } else {
 	set TotalData(scanspersec) {}
 	set TotalData(scantime) {}
@@ -458,7 +537,7 @@ proc redraw_new_style {} {
 	set h [human_number [expr $relayokspersec_total * 3600]]
 	set d [human_number [expr $relayokspersec_total * 86400]]
 	.top.relayspersectotal configure -text [format "%.2f\n%s/h\n%s/d" $relayokspersec_total $h $d]
-	.top.avgrelaytime configure -text [format "%.0f" [expr $relayoktime_total / (1.0 * $index)]]
+	.top.avgrelaytime configure -text [format "%.0f" $relayoktime_avg]
     } else {
 	set TotalData(relayokspersec) {}
 	set TotalData(relayoktime) {}
@@ -471,7 +550,7 @@ proc redraw_new_style {} {
 	set h [human_number [expr $senderokspersec_total * 3600]]
 	set d [human_number [expr $senderokspersec_total * 86400]]
 	.top.senderspersectotal configure -text [format "%.2f\n%s/h\n%s/d" $senderokspersec_total $h $d]
-	.top.avgsendertime configure -text [format "%.0f" [expr $senderoktime_total / (1.0 * $index)]]
+	.top.avgsendertime configure -text [format "%.0f" $senderoktime_avg]
 	set s [human_number $senderokspersec_total]
     } else {
 	set TotalData(senderokspersec) {}
@@ -485,7 +564,7 @@ proc redraw_new_style {} {
 	set h [human_number [expr $recipokspersec_total * 3600]]
 	set d [human_number [expr $recipokspersec_total * 86400]]
 	.top.recipspersectotal configure -text [format "%.2f\n%s/h\n%s/d" $recipokspersec_total $h $d]
-	.top.avgreciptime configure -text [format "%.0f" [expr $recipoktime_total / (1.0 * $index)]]
+	.top.avgreciptime configure -text [format "%.0f" $recipoktime_avg]
     } else {
 	set TotalData(recipokspersec) {}
 	set TotalData(recipoktime) {}
@@ -763,6 +842,8 @@ proc reconfigure_new_style {} {
     label .top.busy -text "Busy Slaves   " -foreground "#A00000"
     grid .top.name -row 0 -column 0 -sticky new
     grid .top.busy -row 0 -column 1 -sticky new
+    catch { destroy .top.clabels}
+    canvas .top.clabels -width $canv_width -height 10 -takefocus 0 -borderwidth 0 -background #ffffff -highlightthickness 0
     if {$NewStyleShowScans} {
 	incr canv_width 200
 	label .top.scanspersec -text "Scans/s   " -foreground "#00A000"
@@ -800,6 +881,8 @@ proc reconfigure_new_style {} {
 	incr col
     }
 
+    grid .top.clabels -row 0 -column $col -sticky nsew
+
     set index 0
     foreach m $Machines {
 	grid_machine_new_style $m $index
@@ -837,41 +920,43 @@ proc reconfigure_new_style {} {
 
     set col 2
     label .top.totalrow -text "Totals:"
-    label .top.busytotal
+    label .top.busytotal -foreground "#A00000"
     grid .top.totalrow -row $row -column 0 -sticky new
     grid .top.busytotal -row $row -column 1 -sticky new
     if {$NewStyleShowScans} {
-	label .top.scanspersectotal
+	label .top.scanspersectotal -foreground "#00A000"
 	grid .top.scanspersectotal -row $row -column $col -sticky new
 	incr col
-	label .top.avgscantime
+	label .top.avgscantime -foreground "#0000A0"
 	grid .top.avgscantime -row $row -column $col -sticky new
 	incr col
     }
     if {$NewStyleShowRelayoks} {
-	label .top.relayspersectotal
+	label .top.relayspersectotal -foreground "#808000"
 	grid .top.relayspersectotal -row $row -column $col -sticky new
 	incr col
-	label .top.avgrelaytime
+	label .top.avgrelaytime -foreground "#008080"
 	grid .top.avgrelaytime -row $row -column $col -sticky new
 	incr col
     }
     if {$NewStyleShowSenderoks} {
-	label .top.senderspersectotal
+	label .top.senderspersectotal -foreground "#808080"
 	grid .top.senderspersectotal -row $row -column $col -sticky new
 	incr col
-	label .top.avgsendertime
+	label .top.avgsendertime -foreground "#800080"
 	grid .top.avgsendertime -row $row -column $col -sticky new
 	incr col
     }
     if {$NewStyleShowRecipoks} {
-	label .top.recipspersectotal
+	label .top.recipspersectotal -foreground "#008000"
 	grid .top.recipspersectotal -row $row -column $col -sticky new
 	incr col
-	label .top.avgreciptime
+	label .top.avgreciptime -foreground "#000000"
 	grid .top.avgreciptime -row $row -column $col -sticky new
 	incr col
     }
+    set num_items [expr $col-1]
+
     canvas .top.c$index -width $canv_width -height 60 -takefocus 0 -borderwidth 0 -background #FFFFEE -highlightthickness 0
     grid .top.c$index -row $row -column $col -sticky nsew -pady 1
     grid rowconfigure .top $row -weight 3
@@ -1148,6 +1233,10 @@ proc grid_machine_new_style { m index } {
 	set disp_m $host
     }
 
+    # Chop off domain name from host
+    if {[regexp {^([^.]+)\.} $disp_m foo new_m]} {
+	set disp_m $new_m
+    }
     set row [expr $index + 1]
     catch { destroy .top.name$index }
     catch { destroy .top.busy$index }
@@ -1167,42 +1256,42 @@ proc grid_machine_new_style { m index } {
     bind .top.name$index <ButtonPress-1> [list popup_machine_menu $m $index %X %Y]
     bind .top.name$index <ButtonPress-2> [list popup_machine_menu $m $index %X %Y]
     bind .top.name$index <ButtonPress-3> [list popup_machine_menu $m $index %X %Y]
-    label .top.busy$index -text ""
+    label .top.busy$index -text "" -foreground "#A00000"
     grid .top.name$index -row $row -column 0 -sticky new
     grid .top.busy$index -row $row -column 1 -sticky new
 
     if {$NewStyleShowScans} {
-	label .top.scanspersec$index
+	label .top.scanspersec$index -foreground "#00A000"
 	grid .top.scanspersec$index -row $row -column $column -sticky new
 	incr column
-	label .top.scantime$index
+	label .top.scantime$index -foreground "#0000A0"
 	grid .top.scantime$index -row $row -column $column -sticky new
 	incr column
 	incr canv_width 200
     }
     if {$NewStyleShowRelayoks} {
-	label .top.relayspersec$index
+	label .top.relayspersec$index -foreground "#808000"
 	grid .top.relayspersec$index -row $row -column $column -sticky new
 	incr column
-	label .top.relaytime$index
+	label .top.relaytime$index -foreground "#008080"
 	grid .top.relaytime$index -row $row -column $column -sticky new
 	incr column
 	incr canv_width 200
     }
     if {$NewStyleShowSenderoks} {
-	label .top.senderspersec$index
+	label .top.senderspersec$index -foreground "#808080"
 	grid .top.senderspersec$index -row $row -column $column -sticky new
 	incr column
-	label .top.sendertime$index
+	label .top.sendertime$index -foreground "#800080"
 	grid .top.sendertime$index -row $row -column $column -sticky new
 	incr column
 	incr canv_width 200
     }
     if {$NewStyleShowRecipoks} {
-	label .top.recipspersec$index
+	label .top.recipspersec$index -foreground "#008000"
 	grid .top.recipspersec$index -row $row -column $column -sticky new
 	incr column
-	label .top.reciptime$index
+	label .top.reciptime$index -foreground "#000000"
 	grid .top.reciptime$index -row $row -column $column -sticky new
 	incr column
 	incr canv_width 200
